local UIParent, insert = UIParent, table.insert;

local Dropdown = nil
local MaxItems = 10
local StartIndex = 0
local DisplayedItem = 1
local NumItems = 1
local Callback_F 

local function CloseDropdown()
    if Dropdown then 
        Dropdown:Release()
        Dropdown = nil
        Callback_F = nil
        StartIndex = 0
    end 
end 

local function ScrollDropdown(Items,Index,Increment)
    if Increment then 
        Index = StartIndex + Index
    end 

    Index = ceil(Index)

    if Index < 1 then 
        Index = 1 
    elseif Index > NumItems - DisplayedItem + 1 then 
        Index = NumItems - DisplayedItem + 1
    end 

    if StartIndex ~= Index then 
        for i = 1,DisplayedItem do
            if Dropdown["Item_"..i] then 
                if Items[Index + i - 1] then 
                    Dropdown["Item_"..i]:SetText(Items[Index + i - 1], 11, "Normal",nil,nil, "LEFT")
                    Dropdown["Item_"..i].OnClick = function()
                        if Callback_F then 
                            Callback_F(Dropdown:GetParent(),Items[Index + i - 1],Index + i - 1)
                        end 
                        CloseDropdown()
                    end
                else 
                    Dropdown["Item_"..i]:SetText("|cFFFF0000X|r Cancel", 11, "Normal",nil,nil, "LEFT")
                    Dropdown["Item_"..i].OnClick = function()
                        CloseDropdown()
                    end
                end 
            end 
        end  
        if Dropdown.SliderButton then 
            Dropdown.SliderButton:SetPoint("TOP", Dropdown.SlidLine, "TOP", 0,-(Index - 1)*20*MaxItems/NumItems)
        end 
        StartIndex = Index
    end
end 

local function UpdateScrollPosition(self)
    local MouseX, MouseY = GetCursorPosition()
    local Scale = UIParent:GetEffectiveScale()
    MouseX = MouseX / Scale
    MouseY = MouseY / Scale

    local Right = Dropdown:GetRight() 

    if MouseX <= Right - self.BorderSize and MouseX >= Right - 20 - self.BorderSize then 
        local dif = Dropdown:GetTop() - self.BorderSize - MouseY
        return dif*NumItems / (20*MaxItems) + 1 - DisplayedItem/2
    end 
end 

function _Avyr_Global_ADDON_Elements_Lib.WidgetsMixin:DropdownCall(Parent , Items, Callback, ForceOpen, Selected)
    Parent = Parent or UIParent
    NumItems = getn(Items) + 1
    if Dropdown and Parent == Dropdown:GetParent() then 
        CloseDropdown()
        return nil
    end 

    if Dropdown then 
        CloseDropdown()
    end 

    Callback_F = Callback

    Dropdown = self:CreateFrame(Parent)
    Dropdown:EnableMouse(true)
    Dropdown:SetFrameStrata("FULLSCREEN_DIALOG")
    Dropdown:SetFrameLevel(8)
    Dropdown:SetThemedBackdrop()

    Dropdown.Content = self:CreateFrame(Dropdown)
    Dropdown.Content:SetPoint("TOPLEFT",self.BorderSize,-self.BorderSize)

    DisplayedItem = NumItems
    if NumItems > MaxItems then 
        Dropdown.SlidLine = self:FetchTexture(Dropdown)
        Dropdown.SlidLine:SetPoint("TOP", Dropdown, "TOPRIGHT",-10-self.BorderSize,-self.BorderSize)
        Dropdown.SlidLine:SetPoint("BOTTOM", Dropdown, "BOTTOMRIGHT",-10-self.BorderSize,self.BorderSize)
        Dropdown.SlidLine:SetWidth(2)
        Dropdown.SlidLine:SetColorTexture(1,1,1,1)
        self:SetThemedColor(Dropdown.SlidLine,"Highlight")
    
        Dropdown.SliderButton = self:CreateFrame(Dropdown)
        Dropdown.SliderButton:Size(20, MaxItems*20*MaxItems/NumItems)
        Dropdown.SliderButton:SetThemedBackdrop()
        Dropdown.SliderButton:SetThemedBackdrop("Backdrop", "Highlight")

        Dropdown:SetScript("OnMouseDown",function()
            Dropdown:SetScript("OnUpdate",function()
                local NewIndex =  UpdateScrollPosition(self)
                if NewIndex then 
                    ScrollDropdown(Items,NewIndex)
                end 
            end)
        end)

        Dropdown:SetScript("OnMouseUp",function()
            Dropdown:SetScript("OnUpdate",nil)
            local NewIndex =  UpdateScrollPosition(self)
            if NewIndex then 
                ScrollDropdown(Items,NewIndex)
            end 
        end)

        Dropdown:SetScript("OnMouseWheel", function(self,delta)
            ScrollDropdown(Items,-delta,true)
        end)
        DisplayedItem = MaxItems
        Dropdown:Size(150,MaxItems*20+2*self.BorderSize)
        Dropdown.Content:SetPoint("BOTTOMRIGHT",-self.BorderSize-20,self.BorderSize)
    else 
        Dropdown:Size(150,NumItems*20+2*self.BorderSize)
        Dropdown.Content:SetPoint("BOTTOMRIGHT",-self.BorderSize,self.BorderSize)
    end  

    for i = 1,DisplayedItem do
        Dropdown["Item_"..i] = self:CreateButton(Dropdown.Content,true)
        Dropdown["Item_"..i]:Size(Width,20)
        Dropdown["Item_"..i]:SetPoint("TOPLEFT",0,-((i - 1)*20))
        Dropdown["Item_"..i]:SetPoint("TOPRIGHT",0,-((i - 1)*20))
        Dropdown["Item_"..i]:SetThemedBackdrop("Clear")
    end 
    ScrollDropdown(Items,1)

    if select(2, UIParent:GetCenter()) < select(2, Parent:GetCenter()) then
        Dropdown:SetPoint("TOPLEFT", Parent, "BOTTOMLEFT", 0, self.BorderSize)
        Dropdown:SetPoint("TOPRIGHT", Parent, "BOTTOMRIGHT", 0, self.BorderSize)
    else
        Dropdown:SetPoint("BOTTOMLEFT", Parent, "TOPLEFT", 0, -self.BorderSize)
        Dropdown:SetPoint("BOTTOMRIGHT", Parent, "TOPRIGHT", 0, -self.BorderSize)
    end
    return Dropdown
end